@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2010, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2011, Live Networks, Inc.  All rights reserved
 //	Help by Carlo Bonamico to get working for Windows
 // Delay queue
 // Implementation
@@ -89,7 +89,7 @@ const DelayInterval ETERNITY(INT_MAX, MILLION-1);
 
 ///// DelayQueueEntry /////
 
-long DelayQueueEntry::tokenCounter = 0;
+intptr_t DelayQueueEntry::tokenCounter = 0;
 
 DelayQueueEntry::DelayQueueEntry(DelayInterval delay)
   : fDeltaTimeRemaining(delay) {
@@ -113,7 +113,11 @@ DelayQueue::DelayQueue()
 }
 
 DelayQueue::~DelayQueue() {
-  while (fNext != this) removeEntry(fNext);
+  while (fNext != this) {
+    DelayQueueEntry* entryToRemove = fNext;
+    removeEntry(entryToRemove);
+    delete entryToRemove;
+  }
 }
 
 void DelayQueue::addEntry(DelayQueueEntry* newEntry) {
@@ -141,7 +145,7 @@ void DelayQueue::updateEntry(DelayQueueEntry* entry, DelayInterval newDelay) {
   addEntry(entry);
 }
 
-void DelayQueue::updateEntry(long tokenToFind, DelayInterval newDelay) {
+void DelayQueue::updateEntry(intptr_t tokenToFind, DelayInterval newDelay) {
   DelayQueueEntry* entry = findEntryByToken(tokenToFind);
   updateEntry(entry, newDelay);
 }
@@ -156,7 +160,7 @@ void DelayQueue::removeEntry(DelayQueueEntry* entry) {
   // in case we should try to remove it again
 }
 
-DelayQueueEntry* DelayQueue::removeEntry(long tokenToFind) {
+DelayQueueEntry* DelayQueue::removeEntry(intptr_t tokenToFind) {
   DelayQueueEntry* entry = findEntryByToken(tokenToFind);
   removeEntry(entry);
   return entry;
@@ -181,7 +185,7 @@ void DelayQueue::handleAlarm() {
   }
 }
 
-DelayQueueEntry* DelayQueue::findEntryByToken(long tokenToFind) {
+DelayQueueEntry* DelayQueue::findEntryByToken(intptr_t tokenToFind) {
   DelayQueueEntry* cur = head();
   while (cur != this) {
     if (cur->token() == tokenToFind) return cur;
