@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "mTunnel" multicast access service
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // Helper routines to implement 'group sockets'
 // Implementation
 
@@ -36,7 +36,7 @@ netAddressBits SendingInterfaceAddr = INADDR_ANY;
 netAddressBits ReceivingInterfaceAddr = INADDR_ANY;
 
 static void socketErr(UsageEnvironment& env, char const* errorMsg) {
-	env.setResultErrMsg(errorMsg);
+  env.setResultErrMsg(errorMsg);
 }
 
 static int reuseFlag = 1;
@@ -49,13 +49,31 @@ NoReuse::~NoReuse() {
   reuseFlag = 1;
 }
 
+static int createSocket(int type) {
+  // Call "socket()" to create a (IPv4) socket of the specified type.
+  // But also set it to have the 'close on exec' property (if we can)
+  int sock;
+
+#ifdef SOCK_CLOEXEC
+  sock = socket(AF_INET, type|SOCK_CLOEXEC, 0);
+  if (sock != -1 || errno != EINVAL) return sock;
+  // An "errno" of EINVAL likely means that the system wasn't happy with the SOCK_CLOEXEC; fall through and try again without it:
+#endif
+
+  sock = socket(AF_INET, type, 0);
+#ifdef FD_CLOEXEC
+  if (sock != -1) fcntl(sock, F_SETFD, FD_CLOEXEC);
+#endif
+  return sock;
+}
+
 int setupDatagramSocket(UsageEnvironment& env, Port port) {
   if (!initializeWinsockIfNecessary()) {
     socketErr(env, "Failed to initialize 'winsock': ");
     return -1;
   }
 
-  int newSocket = socket(AF_INET, SOCK_DGRAM, 0);
+  int newSocket = createSocket(SOCK_DGRAM);
   if (newSocket < 0) {
     socketErr(env, "unable to create datagram socket: ");
     return newSocket;
@@ -129,7 +147,7 @@ int setupDatagramSocket(UsageEnvironment& env, Port port) {
 }
 
 Boolean makeSocketNonBlocking(int sock) {
-#if defined(__WIN32__) || defined(_WIN32) || defined(IMN_PIM)
+#if defined(__WIN32__) || defined(_WIN32)
   unsigned long arg = 1;
   return ioctlsocket(sock, FIONBIO, &arg) == 0;
 #elif defined(VXWORKS)
@@ -142,7 +160,7 @@ Boolean makeSocketNonBlocking(int sock) {
 }
 
 Boolean makeSocketBlocking(int sock) {
-#if defined(__WIN32__) || defined(_WIN32) || defined(IMN_PIM)
+#if defined(__WIN32__) || defined(_WIN32)
   unsigned long arg = 0;
   return ioctlsocket(sock, FIONBIO, &arg) == 0;
 #elif defined(VXWORKS)
@@ -161,7 +179,7 @@ int setupStreamSocket(UsageEnvironment& env,
     return -1;
   }
 
-  int newSocket = socket(AF_INET, SOCK_STREAM, 0);
+  int newSocket = createSocket(SOCK_STREAM);
   if (newSocket < 0) {
     socketErr(env, "unable to create stream socket: ");
     return newSocket;
@@ -222,110 +240,39 @@ int setupStreamSocket(UsageEnvironment& env,
   return newSocket;
 }
 
-#ifndef IMN_PIM
-static int blockUntilReadable(UsageEnvironment& env,
-			      int socket, struct timeval* timeout) {
-  int result = -1;
-  do {
-    fd_set rd_set;
-    FD_ZERO(&rd_set);
-    if (socket < 0) break;
-    FD_SET((unsigned) socket, &rd_set);
-    const unsigned numFds = socket+1;
-
-    result = select(numFds, &rd_set, NULL, NULL, timeout);
-    if (timeout != NULL && result == 0) {
-      break; // this is OK - timeout occurred
-    } else if (result <= 0) {
-      int err = env.getErrno();
-      if (err == EINTR || err == EAGAIN || err == EWOULDBLOCK) continue;
-      socketErr(env, "select() error: ");
-      break;
-    }
-
-    if (!FD_ISSET(socket, &rd_set)) {
-      socketErr(env, "select() error - !FD_ISSET");
-      break;
-    }
-  } while (0);
-
-  return result;
-}
-#else
-extern int blockUntilReadable(UsageEnvironment& env,
-			      int socket, struct timeval* timeout);
-#endif
-
 int readSocket(UsageEnvironment& env,
 	       int socket, unsigned char* buffer, unsigned bufferSize,
-	       struct sockaddr_in& fromAddress,
-	       struct timeval* timeout) {
-  int bytesRead = -1;
-  do {
-    int result = blockUntilReadable(env, socket, timeout);
-    if (timeout != NULL && result == 0) {
-      bytesRead = 0;
-      break;
-    } else if (result <= 0) {
-      break;
-    }
-
-    SOCKLEN_T addressSize = sizeof fromAddress;
-    bytesRead = recvfrom(socket, (char*)buffer, bufferSize, 0,
-			 (struct sockaddr*)&fromAddress,
-			 &addressSize);
-    if (bytesRead < 0) {
-      //##### HACK to work around bugs in Linux and Windows:
-      int err = env.getErrno();
-      if (err == 111 /*ECONNREFUSED (Linux)*/
+	       struct sockaddr_in& fromAddress) {
+  SOCKLEN_T addressSize = sizeof fromAddress;
+  int bytesRead = recvfrom(socket, (char*)buffer, bufferSize, 0,
+			   (struct sockaddr*)&fromAddress,
+			   &addressSize);
+  if (bytesRead < 0) {
+    //##### HACK to work around bugs in Linux and Windows:
+    int err = env.getErrno();
+    if (err == 111 /*ECONNREFUSED (Linux)*/
 #if defined(__WIN32__) || defined(_WIN32)
-	  // What a piece of crap Windows is.  Sometimes
-	  // recvfrom() returns -1, but with an 'errno' of 0.
-	  // This appears not to be a real error; just treat
-	  // it as if it were a read of zero bytes, and hope
-	  // we don't have to do anything else to 'reset'
-	  // this alleged error:
-	  || err == 0
+	// What a piece of crap Windows is.  Sometimes
+	// recvfrom() returns -1, but with an 'errno' of 0.
+	// This appears not to be a real error; just treat
+	// it as if it were a read of zero bytes, and hope
+	// we don't have to do anything else to 'reset'
+	// this alleged error:
+	|| err == 0 || err == EWOULDBLOCK
 #else
-	  || err == EAGAIN
+	|| err == EAGAIN
 #endif
-	  || err == 113 /*EHOSTUNREACH (Linux)*/) {
-			        //Why does Linux return this for datagram sock?
-	fromAddress.sin_addr.s_addr = 0;
-	return 0;
-      }
-      //##### END HACK
-      socketErr(env, "recvfrom() error: ");
-      break;
+	|| err == 113 /*EHOSTUNREACH (Linux)*/) { // Why does Linux return this for datagram sock?
+      fromAddress.sin_addr.s_addr = 0;
+      return 0;
     }
-  } while (0);
+    //##### END HACK
+    socketErr(env, "recvfrom() error: ");
+  }
 
   return bytesRead;
 }
 
-
-int readSocketExact(UsageEnvironment& env,
-		    int socket, unsigned char* buffer, unsigned bufferSize,
-		    struct sockaddr_in& fromAddress,
-		    struct timeval* timeout) {
-  /* read EXACTLY bufferSize bytes from the socket into the buffer.
-     fromaddress is address of last read.
-     return the number of bytes actually read when an error occurs
-  */
-  int bsize = bufferSize;
-  int bytesRead = 0;
-  int totBytesRead =0;
-  do {
-    bytesRead = readSocket (env, socket, buffer + totBytesRead, bsize,
-                            fromAddress, timeout);
-    if (bytesRead <= 0) break;
-    totBytesRead += bytesRead;
-    bsize -= bytesRead;
-  } while (bsize != 0);
-
-  return totBytesRead;
-}
-
 Boolean writeSocket(UsageEnvironment& env,
 		    int socket, struct in_addr address, Port port,
 		    u_int8_t ttlArg,
@@ -550,12 +497,12 @@ Boolean getSourcePort(UsageEnvironment& env, int socket, Port& port) {
   return True;
 }
 
-static Boolean badAddress(netAddressBits addr) {
+static Boolean badAddressForUs(netAddressBits addr) {
   // Check for some possible erroneous addresses:
-  netAddressBits hAddr = ntohl(addr);
-  return (hAddr == 0x7F000001 /* 127.0.0.1 */
-	  || hAddr == 0
-	  || hAddr == (netAddressBits)(~0));
+  netAddressBits nAddr = htonl(addr);
+  return (nAddr == 0x7F000001 /* 127.0.0.1 */
+	  || nAddr == 0
+	  || nAddr == (netAddressBits)(~0));
 }
 
 Boolean loopbackWorks = 1;
@@ -591,85 +538,74 @@ netAddressBits ourIPAddress(UsageEnvironment& env) {
       if (!writeSocket(env, sock, testAddr, testPort, 0,
 		       testString, testStringLength)) break;
 
-      unsigned char readBuffer[20];
+      // Block until the socket is readable (with a 5-second timeout):
+      fd_set rd_set;
+      FD_ZERO(&rd_set);
+      FD_SET((unsigned)sock, &rd_set);
+      const unsigned numFds = sock+1;
       struct timeval timeout;
       timeout.tv_sec = 5;
       timeout.tv_usec = 0;
+      int result = select(numFds, &rd_set, NULL, NULL, &timeout);
+      if (result <= 0) break;
+
+      unsigned char readBuffer[20];
       int bytesRead = readSocket(env, sock,
 				 readBuffer, sizeof readBuffer,
-				 fromAddr, &timeout);
-      if (bytesRead == 0 // timeout occurred
-	  || bytesRead != (int)testStringLength
-	  || strncmp((char*)readBuffer, (char*)testString,
-		     testStringLength) != 0) {
+				 fromAddr);
+      if (bytesRead != (int)testStringLength
+	  || strncmp((char*)readBuffer, (char*)testString, testStringLength) != 0) {
 	break;
       }
 
       loopbackWorks = 1;
     } while (0);
 
+    if (sock >= 0) {
+      socketLeaveGroup(env, sock, testAddr.s_addr);
+      closeSocket(sock);
+    }
+
     if (!loopbackWorks) do {
-      // We couldn't find our address using multicast loopback
-      // so try instead to look it up directly.
+      // We couldn't find our address using multicast loopback,
+      // so try instead to look it up directly - by first getting our host name, and then resolving this host name
       char hostname[100];
       hostname[0] = '\0';
-#ifndef CRIS
-      gethostname(hostname, sizeof hostname);
-#endif
-      if (hostname[0] == '\0') {
+      int result = gethostname(hostname, sizeof hostname);
+      if (result != 0 || hostname[0] == '\0') {
 	env.setResultErrMsg("initial gethostname() failed");
 	break;
       }
 
-#if defined(VXWORKS)
-#include <hostLib.h>
-      if (ERROR == (ourAddress = hostGetByName( hostname ))) break;
-#else
-      struct hostent* hstent
-	= (struct hostent*)gethostbyname(hostname);
-      if (hstent == NULL || hstent->h_length != 4) {
-	env.setResultErrMsg("initial gethostbyname() failed");
-	break;
-      }
-      // Take the first address that's not bad
-      // (This code, like many others, won't handle IPv6)
-      netAddressBits addr = 0;
-      for (unsigned i = 0; ; ++i) {
-	char* addrPtr = hstent->h_addr_list[i];
-	if (addrPtr == NULL) break;
+      // Try to resolve "hostname" to an IP address:
+      NetAddressList addresses(hostname);
+      NetAddressList::Iterator iter(addresses);
+      NetAddress const* address;
 
-	netAddressBits a = *(netAddressBits*)addrPtr;
-	if (!badAddress(a)) {
+      // Take the first address that's not bad:
+      netAddressBits addr = 0;
+      while ((address = iter.nextAddress()) != NULL) {
+	netAddressBits a = *(netAddressBits*)(address->data());
+	if (!badAddressForUs(a)) {
 	  addr = a;
 	  break;
 	}
       }
-      if (addr != 0) {
-	fromAddr.sin_addr.s_addr = addr;
-      } else {
-	env.setResultMsg("no address");
-	break;
-      }
+
+      // Assign the address that we found to "fromAddr" (as if the 'loopback' method had worked), to simplify the code below: 
+      fromAddr.sin_addr.s_addr = addr;
     } while (0);
 
     // Make sure we have a good address:
     netAddressBits from = fromAddr.sin_addr.s_addr;
-    if (badAddress(from)) {
+    if (badAddressForUs(from)) {
       char tmp[100];
-      sprintf(tmp,
-	      "This computer has an invalid IP address: 0x%x",
-	      (netAddressBits)(ntohl(from)));
+      sprintf(tmp, "This computer has an invalid IP address: %s", AddressString(from).val());
       env.setResultMsg(tmp);
       from = 0;
     }
 
     ourAddress = from;
-#endif
-
-    if (sock >= 0) {
-      socketLeaveGroup(env, sock, testAddr.s_addr);
-      closeSocket(sock);
-    }
 
     // Use our newly-discovered IP address, and the current time,
     // to initialize the random number generator's seed:
@@ -690,7 +626,7 @@ netAddressBits chooseRandomIPv4SSMAddress(UsageEnvironment& env) {
   netAddressBits const first = 0xE8000100, lastPlus1 = 0xE8FFFFFF;
   netAddressBits const range = lastPlus1 - first;
 
-  return htonl(first + ((netAddressBits)our_random())%range);
+  return ntohl(first + ((netAddressBits)our_random())%range);
 }
 
 char const* timestampString() {
@@ -700,12 +636,16 @@ char const* timestampString() {
 #if !defined(_WIN32_WCE)
   static char timeString[9]; // holds hh:mm:ss plus trailing '\0'
   char const* ctimeResult = ctime((time_t*)&tvNow.tv_sec);
-  char const* from = &ctimeResult[11];
-  int i;
-  for (i = 0; i < 8; ++i) {
-    timeString[i] = from[i];
+  if (ctimeResult == NULL) {
+    sprintf(timeString, "??:??:??");
+  } else {
+    char const* from = &ctimeResult[11];
+    int i;
+    for (i = 0; i < 8; ++i) {
+      timeString[i] = from[i];
+    }
+    timeString[i] = '\0';
   }
-  timeString[i] = '\0';
 #else
   // WinCE apparently doesn't have "ctime()", so instead, construct
   // a timestamp string just using the integer and fractional parts
@@ -717,7 +657,7 @@ char const* timestampString() {
   return (char const*)&timeString;
 }
 
-#if (defined(__WIN32__) || defined(_WIN32)) && !defined(IMN_PIM)
+#if defined(__WIN32__) || defined(_WIN32)
 // For Windoze, we need to implement our own gettimeofday()
 #if !defined(_WIN32_WCE)
 #include <sys/timeb.h>
