@@ -35,7 +35,7 @@ MPEG1or2AudioRTPSink::createNew(UsageEnvironment& env, Groupsock* RTPgs) {
 void MPEG1or2AudioRTPSink::doSpecialFrameHandling(unsigned fragmentationOffset,
 					      unsigned char* frameStart,
 					      unsigned numBytesInFrame,
-					      struct timeval frameTimestamp,
+					      struct timeval framePresentationTime,
 					      unsigned numRemainingBytes) {
   // If this is the 1st frame in the 1st packet, set the RTP 'M' (marker)
   // bit (because this is considered the start of a talk spurt):
@@ -53,7 +53,7 @@ void MPEG1or2AudioRTPSink::doSpecialFrameHandling(unsigned fragmentationOffset,
   // to set the packet's timestamp:
   MultiFramedRTPSink::doSpecialFrameHandling(fragmentationOffset,
 					     frameStart, numBytesInFrame,
-					     frameTimestamp,
+					     framePresentationTime,
 					     numRemainingBytes);
 }
 
