@@ -376,7 +376,7 @@ int RTSPClient::openConnection() {
     if (fInputSocketNum < 0) break;
       
     // Connect to the remote endpoint:
-    fServerAddress = *(unsigned*)(destAddress.data());
+    fServerAddress = *(netAddressBits*)(destAddress.data());
     int connectResult = connectToServer(fInputSocketNum, destPortNum);
     if (connectResult < 0) break;
     else if (connectResult > 0) {
@@ -394,7 +394,7 @@ int RTSPClient::openConnection() {
 int RTSPClient::connectToServer(int socketNum, portNumBits remotePortNum) {
   MAKE_SOCKADDR_IN(remoteName, fServerAddress, htons(remotePortNum));
   if (fVerbosityLevel >= 1) {
-    envir() << "Opening connection to " << our_inet_ntoa(remoteName.sin_addr) << ", port " << remotePortNum << "...\n";
+    envir() << "Opening connection to " << AddressString(remoteName).val() << ", port " << remotePortNum << "...\n";
   }
   if (connect(socketNum, (struct sockaddr*) &remoteName, sizeof remoteName) != 0) {
     if (envir().getErrno() == EINPROGRESS) {
@@ -469,7 +469,7 @@ static char* createScaleString(float scale, float currentScale) {
     // This is the default value; we don't need a "Scale:" header:
     buf[0] = '\0';
   } else {
-    Locale l("C", LC_NUMERIC);
+    Locale l("C", Numeric);
     sprintf(buf, "Scale: %f\r\n", scale);
   }
 
@@ -483,11 +483,11 @@ static char* createRangeString(double start, double end) {
     buf[0] = '\0';
   } else if (end < 0) {
     // There's no end time:
-    Locale l("C", LC_NUMERIC);
+    Locale l("C", Numeric);
     sprintf(buf, "Range: npt=%.3f-\r\n", start);
   } else {
     // There's both a start and an end time; include them both in the "Range:" hdr
-    Locale l("C", LC_NUMERIC);
+    Locale l("C", Numeric);
     sprintf(buf, "Range: npt=%.3f-%.3f\r\n", start, end);
   }
 
@@ -921,7 +921,7 @@ Boolean RTSPClient::parseTransportParams(char const* paramsStr,
 }
 
 Boolean RTSPClient::parseScaleParam(char const* paramStr, float& scale) {
-  Locale l("C", LC_NUMERIC);
+  Locale l("C", Numeric);
   return sscanf(paramStr, "%f", &scale) == 1;
 }
 
@@ -1713,11 +1713,15 @@ char* RTSPClient::sendOptionsCmd(char const* url,
       haveAllocatedAuthenticator = True;
       
       result = sendOptionsCmd(url, username, password, authenticator, timeout);
-      if (result != NULL) return result; // We are already authorized
+      if (result != NULL) { // We are already authorized
+	delete authenticator;
+	return result;
+      }
 
       // The "realm" field should have been filled in:
       if (authenticator->realm() == NULL) {
 	// We haven't been given enough information to try again, so fail:
+	delete authenticator;
 	return NULL;
       }
     }
@@ -1733,6 +1737,7 @@ char* RTSPClient::sendOptionsCmd(char const* url,
     fTimeoutTask = envir().taskScheduler().scheduleDelayedTask(timeout*1000000, timeoutHandlerForSyncInterface, this);
   }
   (void)sendOptionsCommand(responseHandlerForSyncInterface, authenticator);
+  if (haveAllocatedAuthenticator) delete authenticator;
 
   // Now block (but handling events) until we get a response (or a timeout):
   envir().taskScheduler().doEventLoop(&fWatchVariableForSyncInterface);
