@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2010, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2011, Live Networks, Inc.  All rights reserved
 // A common framework, used for the "openRTSP" and "playSIP" applications
 // Implementation
 
@@ -310,8 +310,7 @@ int main(int argc, char** argv) {
 	}
       }
 
-      ourAuthenticator = new Authenticator;
-      ourAuthenticator->setUsernameAndPassword(username, password);
+      ourAuthenticator = new Authenticator(username,password);
       break;
     }
 
@@ -766,8 +765,7 @@ void setupStreams() {
 	  // Also set a handler to be called if a RTCP "BYE" arrives
 	  // for this subsession:
 	  if (subsession->rtcpInstance() != NULL) {
-	    subsession->rtcpInstance()->setByeHandler(subsessionByeHandler,
-						      subsession);
+	    subsession->rtcpInstance()->setByeHandler(subsessionByeHandler, subsession);
 	  }
 
 	  madeProgress = True;
@@ -814,6 +812,10 @@ void continueAfterPLAY(RTSPClient*, int resultCode, char* resultString) {
   Boolean timerIsBeingUsed = False;
   double secondsToDelay = duration;
   if (duration > 0) {
+    // First, adjust "duration" based on any change to the play range (that was specified in the "PLAY" response):
+    double rangeAdjustment = (session->playEndTime() - session->playStartTime()) - (endTime - initialSeekTime);
+    if (duration + rangeAdjustment > 0.0) duration += rangeAdjustment;
+
     timerIsBeingUsed = True;
     double absScale = scale > 0 ? scale : -scale; // ASSERT: scale != 0
     secondsToDelay = duration/absScale + durationSlop;
@@ -894,7 +896,16 @@ void sessionAfterPlaying(void* /*clientData*/) {
   if (!playContinuously) {
     shutdown(0);
   } else {
-    // We've been asked to play the stream(s) over again:
+    // We've been asked to play the stream(s) over again.
+    // First, reset state from the current session:
+    if (env != NULL) {
+      env->taskScheduler().unscheduleDelayedTask(sessionTimerTask);
+      env->taskScheduler().unscheduleDelayedTask(arrivalCheckTimerTask);
+      env->taskScheduler().unscheduleDelayedTask(interPacketGapCheckTimerTask);
+      env->taskScheduler().unscheduleDelayedTask(qosMeasurementTimerTask);
+    }
+    totNumPacketsReceived = ~0;
+
     startPlayingSession(session, initialSeekTime, endTime, scale, continueAfterPLAY);
   }
 }
@@ -1112,8 +1123,12 @@ void printQOSData(int exitCode) {
   delete qosRecordHead;
 }
 
+Boolean areAlreadyShuttingDown = False;
 int shutdownExitCode;
 void shutdown(int exitCode) {
+  if (areAlreadyShuttingDown) return; // in case we're called after receiving a RTCP "BYE" while in the middle of a "TEARDOWN".
+  areAlreadyShuttingDown = True;
+
   shutdownExitCode = exitCode;
   if (env != NULL) {
     env->taskScheduler().unscheduleDelayedTask(sessionTimerTask);
